package dk.brics.bisim.controller;

import java.rmi.Remote;
import java.rmi.RemoteException;

/**
 * A local controller listening on view events that may have
 * global effects. It informs a global message passer of such events, and it receives
 * commands from the message passer in response to remote game events.
 * 
 * @author Martin Mosegaard <a href="mailto:martinm@daimi.au.dk">martinm@daimi.au.dk</a>
 */
public interface GameController extends Remote {

	/**
	 * Invoked by the message passer when status has changed by another players
	 * move.
	 * Message numbers are specified in the protocol.
	 * @param messageNumber A protocol message number
	 * @param message A message
	 * @throws RemoteException
	 * @see dk.brics.bisim.model.Protocol
	 */
	public void updateStatus(int messageNumber, String message) throws RemoteException;
	
	/**
	 * Invoked by the message passer when the other player wants to quit.
	 * @throws RemoteException
	 */
	public void quit() throws RemoteException;

	/**
	 * Invoked by the message passer when the other player sends a 
	 * chat message.
	 * @param message
	 * @throws RemoteException
	 */
	public void setChatMessage(String message) throws RemoteException;
}
