package dk.brics.bisim.controller;

import java.rmi.Remote;
import java.rmi.RemoteException;

/**
 * A global class passing messages between players.
 * @author Martin Mosegaard <a href="mailto:martinm@daimi.au.dk">martinm@daimi.au.dk</a>
 *
 */
public interface MessagePasser extends Remote {

	/**
	 * Registers a local game controller, such that it may
	 * receive messages.
	 * Will register at most two game controllers.
	 * @param c A game controller
	 * @param clientServerRole The role of the game controller
	 * @throws RemoteException
	 */
	public void registerGameController(GameController c, int clientServerRole) throws RemoteException;
	
	/**
	 * Used for sending move messages between game controllers.
	 * @param messageNumber A message number 	  
	 * @param message An encoded message
	 * @see dk.brics.bisim.model.Protocol
	 * @throws RemoteException
	 */
	public void sendMessage(int messageNumber, String message) throws RemoteException;
		
	/**
	 * Notifies listeners that a player wants to quit, and unregisters 
	 * that player.
	 * @throws RemoteException
	 */
	public void quit(int clientServerRole) throws RemoteException;
	
	/**
	 * Used for sending chat messages to the other player.
	 * @param clientServerRole The role of the player sending a message, used to identify 
	 * the other
	 * @param message The message
	 * @throws RemoteException
	 */
	public void talk(int clientServerRole, String message) throws RemoteException;
}
