
package dk.brics.bisim.controller;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.HashMap;

import dk.brics.bisim.model.OptionsModel;
import dk.brics.bisim.model.Protocol;



/**
 * Implementation of the MessagePasser.
 * @see MessagePasser
 * @author Martin Mosegaard <a href="mailto:martinm@daimi.au.dk">martinm@daimi.au.dk</a>
 *
 */
public class MessagePasserImpl extends UnicastRemoteObject
                               implements MessagePasser {

	private static final long serialVersionUID = 1L;
	
	/**
	 * Registrered game controllers. 
	 */
	private HashMap<Integer,GameController> gameControllers;
	
	/**
	 * Constructs a message passer with a map of game controllers, identified
	 * by their client server role.
	 * @throws RemoteException
	 */
	public MessagePasserImpl() throws RemoteException {
		gameControllers = new HashMap<Integer,GameController>();
	}

	public synchronized void registerGameController(GameController c, int clientServerRole) throws RemoteException {
		if (gameControllers.size() == 1 && clientServerRole == OptionsModel.CLIENT) {
			gameControllers.put(clientServerRole, c);
			for (GameController com : gameControllers.values())
				com.updateStatus(Protocol.MESSAGE_RESET_GAME, "");
		}
		else if (gameControllers.size() == 0) {
			gameControllers.put(clientServerRole, c);
			if (clientServerRole == OptionsModel.LOCAL)
				c.updateStatus(Protocol.MESSAGE_RESET_GAME, "");
		}
		else System.out.println("Cannot register communicator: " + c.toString());

	}
	
	public synchronized void sendMessage(int messageNumber, String message) throws RemoteException {
		for (GameController c : gameControllers.values())
			c.updateStatus(messageNumber, message);	
	}
	
	public synchronized void quit(int clientServerRole) throws RemoteException {
		/*** Unregister the player that wants to quit ***/
		if (gameControllers.remove(clientServerRole) != null) 
			System.out.println("[Player quit]");	
		
		/*** Then notify the other ***/
		for (GameController com : gameControllers.values())
			com.quit();	
	}

	public void talk(int clientServerRole, String message) throws RemoteException {
		GameController c = null;
		
		/*** Get the other player ***/
		if (clientServerRole == OptionsModel.CLIENT)
			c = gameControllers.get(OptionsModel.SERVER);
		else if (clientServerRole == OptionsModel.SERVER)
			c = gameControllers.get(OptionsModel.CLIENT);
		
		if (c != null) 
			c.setChatMessage(message);
	}
	


	
}
