package dk.brics.bisim.equiv;

import att.grappa.Node;

/**
 * An abstract successor represents an outgoing edge and a to-node.
 * @author Martin Mosegaard <a href="mailto:martinm@daimi.au.dk">martinm@daimi.au.dk</a>
 *
 */
public abstract class AbstractSuccessor {

	protected String edgeLabel;
	protected Node toNode;
	protected boolean isWeak;
	
	/**
	 * Constructs an abstract successor and sets the fields.
	 * @param edgeLabel The label of the edge
	 * @param toNode The to-node
	 * @param isWeak If this successor is weak
	 */
	public AbstractSuccessor(String edgeLabel, Node toNode, boolean isWeak) {
		this.edgeLabel = edgeLabel;
		this.toNode = toNode;
		this.isWeak = isWeak;
	}
	
	/**
	 * @return The edge label
	 */
	public String getEdgeLabel() {
		return edgeLabel;
	}
	
	/**
	 * @return If the successor is weak
	 */
	public boolean isWeak() {
		return isWeak;
	}

	/**
	 * @return The to-node
	 */
	public Node getToNode() {
		return toNode;
	}

}
