package dk.brics.bisim.equiv;

import java.util.Set;
import java.util.ArrayList;

import att.grappa.Node;

/**
 * A partition in a state space.
 * It contains a set of nodes, and it is identified by a block number.
 * @author Martin Mosegaard <a href="mailto:martinm@daimi.au.dk">martinm@daimi.au.dk</a>
 * @see dk.brics.bisim.equiv.StateSpace
 *
 */
public class Block {

	private ArrayList<Node> nodes;
	private int blockNumber;
	
	/**
	 * Constructs a block holding a set of nodes and identified
	 * by a block number.
	 * @param nodes A set of nodes
	 * @param blockNumber A block number
	 */
	public Block(Set<Node> nodes, int blockNumber) {
		this.nodes = new ArrayList<Node>(nodes);		
		this.blockNumber = blockNumber;
	}	
	
	/**
	 * Returns the number of this block.
	 * @return The number of this block
	 */
	public int getBlockNumber() {
		return blockNumber;
	}

	/**
	 * Returns the nodes in this block.
	 * @return The nodes in this block
	 */
	public ArrayList<Node> getNodes() {
		return nodes;
	}
	
	public String toString() {
		StringBuilder sb = new StringBuilder("[Block #");
		sb.append(blockNumber).append(" { ");
		
		for (Node node : nodes) 
			sb.append("(").append(node.getName()).append(") ");
		
		sb.append("} ]");
		return sb.toString();		
	}
		
}
