package dk.brics.bisim.equiv;

import att.grappa.Node;

/**
 * A node in a configuration graph.
 * @author Martin Mosegaard <a href="mailto:martinm@daimi.au.dk">martinm@daimi.au.dk</a>
 *
 */
public class CGNode {

	private Node left, right;
	private int id;
	private int distance;
	
	/**
	 * Used for creating the accept node.
	 * @param id Node ID
	 */
	public CGNode(int id) {
		left  = null;
		right = null;
		this.id = id;
		distance = 0;
	}
	
	/**
	 * Constructs a configuration.
	 * @param left The left node
	 * @param right The right node
	 * @param id The ID of this configuration node
	 */
	public CGNode(Node left, Node right, int id) {
		this.left = left;
		this.right = right;
		this.id = id;
		distance = -1;
	}

	/**
	 * Returns the left node.
	 * @return The left node
	 */
	public Node getLeft() {
		return left;
	}

	/**
	 * Returns the right node.
	 * @return The right node
	 */
	public Node getRight() {
		return right;
	}
	
	public boolean equals(Object other) {
		if (other instanceof CGNode) 
			return equals((CGNode) other);
		else return false;
	}
	
	private boolean equals(CGNode other) {
		return id == other.getId();
	}
	
	public int hashCode() {
		if (left == null && right == null) 
			return id;
		else 
			return 37 * (id + left.getName().hashCode() + right.getName().hashCode());
	}

	/**
	 * Returns the ID.
	 * @return The ID
	 */
	public int getId() {
		return id;
	}

	public String toString() {
		StringBuilder sb = new StringBuilder("[CGNode #");
		
		sb.append(id).append(' ').append(left.getName())
		.append(',').append(right.getName()).append(" dist=")
		.append(distance).append("]");
		
		return sb.toString();
	}

	public int getDistance() {
		return distance;
	}

	public void setDistance(int distance) {
		this.distance = distance;
	}
	
}
