package dk.brics.bisim.equiv;

import att.grappa.Node;

/**
 * The capability of a node.<p>
 * 
 * A capability records, for a node, an outgoing action, the target block number
 * and whether the transition is weak, i.e., if it has been created during the
 * creation of a weak successor map.
 * @author Martin Mosegaard <a href="mailto:martinm@daimi.au.dk">martinm@daimi.au.dk</a>
 *
 */
public class Capability extends AbstractSuccessor {

	private int blockNumber;
	
	/**
	 * Constructs a capability for a node.
	 * @param edgeLabel The label of the edge
	 * @param toNode The to-node
	 * @param isWeak If this capability is weak, i.e. if it
	 * is only included in the weak transitive closure of an action
	 * @param blockNumber The block number of the to-node
	 */
	public Capability(String edgeLabel, Node toNode, boolean isWeak, int blockNumber) {
		super(edgeLabel, toNode, isWeak);
		this.blockNumber = blockNumber;
	}
	
	public boolean equals(Object other) {
		if (other instanceof Capability) 
			return equals((Capability) other);
		else return false;
	}
	
	/**
	 * Two capabilities are equal if there actions are equal
	 * and their blockNumbers are equal.
	 * @param other Other capability
	 * @return If other equals this
	 */
	public boolean equals(Capability other) {
		return edgeLabel.equals(other.getEdgeLabel()) &&
			   blockNumber == other.getBlockNumber();
	}

	/**
	 * Returns the block number.
	 * @return The block number
	 */
	public int getBlockNumber() {
		return blockNumber;
	}

	public String toString() {
		StringBuilder sb = new StringBuilder("(");
		
		sb.append(edgeLabel).append(",").append(blockNumber).append(")");
		
		return sb.toString();
	}

	public int hashCode() {		
		return 37 * (edgeLabel.hashCode() + blockNumber);
	}
	
}
