package dk.brics.bisim.equiv;

import dk.brics.bisim.model.GameModel;

/**
 * An attacker strategy.
 * A strategy consists of a side, an action, and a target node name.
 * @author Martin Mosegaard <a href="mailto:martinm@daimi.au.dk">martinm@daimi.au.dk</a>
 *
 */
public class Strategy {

	private int    side;
	private String action;
	private String targetNodeName;
	
	/**
	 * Constructs a strategy based on a side, an action, and a target node
	 * name.
	 * @param side A side
	 * @param action An action
	 * @param targetNodeName A target node name
	 */
	public Strategy(int side, String action, String targetNodeName) {
		this.side = side;
		this.action = action;
		this.targetNodeName = targetNodeName;
	}

	/**
	 * @return The side of the strategy
	 */
	public int getSide() {
		return side;
	}

	/**
	 * @return The action of the strategy
	 */
	public String getAction() {
		return action;
	}

	/**
	 * @return The target node name of the strategy
	 */
	public String getTargetNodeName() {
		return targetNodeName;
	}	

	public String toString() {
		StringBuilder sb = new StringBuilder("side=");
		
		sb.append((side == GameModel.LEFT_SIDE) ? "left" : "right")
		  .append(" action=").append(action)
		  .append(" targetNodeName=").append(targetNodeName);
		
		return sb.toString();
	}
	
}
