package dk.brics.bisim.equiv;

import java.util.ArrayList;

import att.grappa.Node;

/**
 * The successor of a node. It consists of the edge and the to-node.
 * A weak successor is one that an attacker may not choose.
 * A weak successor does not have a Grappa Edge.
 * Used in successor maps.
 * @author Martin Mosegaard <a href="mailto:martinm@daimi.au.dk">martinm@daimi.au.dk</a>
 *
 */
public class StrongSuccessor extends AbstractSuccessor 
							 implements Successor {

	private String edgeName;
	
	/**
	 * Constructs a strong successor.
	 * @param edgeLabel The label of the edge
	 * @param toNode A to-node
	 */
	public StrongSuccessor(String edgeLabel, String edgeName, Node toNode) {
		super(edgeLabel, toNode, false);
		this.edgeName = edgeName;
	}

	public boolean equals(Object other) {
		if (other instanceof StrongSuccessor) 
			return equals((StrongSuccessor) other);
		else 
			return false;
	}
		
	private boolean equals(StrongSuccessor other) {
		return (edgeLabel.equals(other.getEdgeLabel()) &&
				toNode.equals(other.getToNode()));
	}
	
	public String toString() {
		StringBuilder sb = new StringBuilder("(");
		sb.append(edgeLabel).append(",")
		  .append(toNode.getName()).append(",")
		  .append((isWeak?"weak":"strong")).append(")");
		return sb.toString();
	}
	
	public int hashCode() {
		return 37 * (edgeLabel.hashCode() + toNode.getName().hashCode());
	}

	public ArrayList<String> getEdgeNames() {
		ArrayList<String> names = new ArrayList<String>();
		names.add(edgeName);
		return names;
	}

}
