package dk.brics.bisim.equiv;

import java.util.ArrayList;

import att.grappa.Node;

/**
 * A weak successor. It contains a list of edge names which are useful for coloring
 * edges.
 * @author Martin Mosegaard <a href="mailto:martinm@daimi.au.dk">martinm@daimi.au.dk</a>
 *
 */
public class WeakSuccessor extends AbstractSuccessor 
                           implements Successor {

	private ArrayList<String> edgeNames;
	
	public WeakSuccessor(String edgeLabel, Node toNode) {
		super(edgeLabel, toNode, true);
		edgeNames = new ArrayList<String>();
	}

	public void addEdgeNames(ArrayList<String> names) {
		edgeNames.addAll(names);
	}

	public ArrayList<String> getEdgeNames() {
		return edgeNames;
	}
	
	public boolean equals(Object other) {
		if (other instanceof WeakSuccessor) 
			return equals((WeakSuccessor) other);
		else 
			return false;
	}
		
	private boolean equals(WeakSuccessor other) {
		return (edgeLabel.equals(other.getEdgeLabel()) &&
				toNode.equals(other.getToNode()));
	}
	
	public String toString() {
		StringBuilder sb = new StringBuilder("(");
		sb.append(edgeLabel).append(",")
		  .append(toNode.getName()).append(",weak)");
		return sb.toString();
	}
	
	public int hashCode() {
		return 37 * (edgeLabel.hashCode() + toNode.getName().hashCode());
	}

}
