package dk.brics.bisim.model;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashSet;

/**
 * A data model for making dot representations from graph files.
 * @author Martin Mosegaard <a href="mailto:martinm@daimi.au.dk">martinm@daimi.au.dk</a>
 *
 */
public class DotModel {

	/**
	 * From a file containing graph output from The Edinburgh CWB,
	 * a dot graph is constructed. This is later passed as input 
	 * to Grappa. 
	 * @param filename File name
	 * @return Dot representation of a graph
	 */
	public String getDotFromGraph (String filename) {
		BufferedReader reader = null;
		try {
			reader = new BufferedReader (new FileReader (filename));		
		} catch (FileNotFoundException fnf) {
			System.err.println (fnf.toString ());
			System.exit (1);
		}
		
		StringBuffer b = new StringBuffer("digraph Automaton {\n");
		b.append("  graph [];\n  node [ style = filled ];\n");
		String str, transitionString, currentState, nextState;
		HashSet<String> states = new HashSet<String>();
		
		try {
			/***** Read initial empty line *****/
			str = reader.readLine();
			/***** Each iteration reads one state *****/
			while (str != null) { 
				/***** Read state name *****/
				currentState = reader.readLine();
				states.add(currentState);
				/***** Read empty line *****/
				reader.readLine();
				/***** Read outgoing transitions *****/
				while ((str = reader.readLine()) != null &&
						str.length() != 0) {
					if (str.startsWith("---")) {
						transitionString = str.substring(4, str.indexOf(" --->"));
						nextState = str.substring(str.indexOf(">") + 2);
						/***** Write transition *****/
						b.append("  \"")
						 .append(currentState)
						 .append("\" -> \"")
						 .append(nextState)
						 .append("\" [label=\"")
						 .append(transitionString)
						 .append("\"]\n");
					}
				}
			}
			reader.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		/***** Write states *****/
		for (String s : states) {
			b.append("  \"").append(s).append("\" [shape=circle,label=\"\",color=white];\n");
		}
		
		b.append("}\n");
		return b.toString();
	}

}
