package dk.brics.bisim.model;

import java.rmi.Remote;
import java.rmi.RemoteException;

/**
 * The remote interface of a global model.
 * Defines methods that can be invoked remotely using RMI.
 * These methods are those invoked by any GameController instance.
 * 
 * @author Martin Mosegaard <a href="mailto:martinm@daimi.au.dk">martinm@daimi.au.dk</a>
 *
 */
public interface GlobalModel extends Remote {

	/**
	 * @return The left file name
	 * @throws RemoteException
	 */
	public String getLeftFile() throws RemoteException;

	/**
	 * @return The right file name
	 * @throws RemoteException
	 */
	public String getRightFile() throws RemoteException;

	/**
	 * @return The role of the player who created the global model object
	 * (attacker/defender)
	 * @throws RemoteException
	 */
	public int getRole() throws RemoteException;
	
	/**
	 * @return The left Grappa graph as a dot representation 
	 * @throws RemoteException
	 */
	public String getLeftGraphAsDot() throws RemoteException;

	/**
	 * @return The right Grappa graph as a dot representation
	 * @throws RemoteException
	 */
	public String getRightGraphAsDot() throws RemoteException;
	
	/**
	 * @return The bisimulation type (strong/weak)
	 * @throws RemoteException
	 */
	public int getBisimulation() throws RemoteException;
}
