package dk.brics.bisim.model;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

/**
 * Implementation of the GlobalModel.
 * @see GlobalModel
 * @author Martin Mosegaard <a href="mailto:martinm@daimi.au.dk">martinm@daimi.au.dk</a>
 *
 */
public class GlobalModelImpl extends UnicastRemoteObject
                             implements GlobalModel {

	private static final long serialVersionUID = 1L;
	
	private String leftFile, leftGraphAsDot,
	               rightFile, rightGraphAsDot;
	private int role, bisimulation;
	
	/**
	 * Constructs a global model.
	 * @param leftFilePath Path name of the left LTS
	 * @param rightFilePath Path name of the right LTS
	 * @param leftFile File name of the left LTS
	 * @param rightFile File name of the right LTS
	 * @param role The role of the player creating the global model
	 * @param bisimulation The bisimulation type
	 * @throws RemoteException
	 */
	public GlobalModelImpl(String leftFilePath, 
						   String rightFilePath,
						   String leftFile,
						   String rightFile,
						   int role,
						   int bisimulation,
						   DotModel dotModel) throws RemoteException {
		
		this.leftFile = leftFile;
		this.rightFile = rightFile;
		this.role = role;
		this.bisimulation = bisimulation;
		
		/*** Create dot representations of files ***/
		leftGraphAsDot = dotModel.getDotFromGraph(leftFilePath);
		rightGraphAsDot = dotModel.getDotFromGraph(rightFilePath);
	}

	public String getLeftFile() throws RemoteException {
		return leftFile;
	}

	public String getRightFile() throws RemoteException {
		return rightFile;
	}

	public int getRole() throws RemoteException {
		return role;
	}

	public String getLeftGraphAsDot() throws RemoteException {
		return leftGraphAsDot;
	}

	public String getRightGraphAsDot() throws RemoteException {
		return rightGraphAsDot;
	}

	public int getBisimulation() throws RemoteException {
		return bisimulation;
	}
	
}
