package dk.brics.bisim.model;

import java.util.StringTokenizer;
import java.util.ArrayList;

/**
 * Static information for remote method invocation. 
 * 
 * @author Martin Mosegaard <a href="mailto:martinm@daimi.au.dk">martinm@daimi.au.dk</a>
 *
 */
public class Protocol {

	/**
	 * A fixed port on which to create a registry. 
	 */
	public static final int PORT = 1299;
	
	/**
	 * The registry name of the global model object.
	 */
	public static final String GLOBAL_MODEL_NAME   = "GlobalModel";
	
	/**
	 * The registry name of the message passer object.
	 */
	public static final String MESSAGE_PASSER_NAME = "MessagePasser";
	
	public static final String EX_MESSAGE_UNBOUND  = "unbound";
	
	public static final int MESSAGE_ATTACKER_CHOSE_SIDE = 1;
	public static final int MESSAGE_ATTACKER_STEP       = 2;
	public static final int MESSAGE_DEFENDER_STEP       = 3;
	public static final int MESSAGE_RESET_GAME          = 4;

	/**
	 * A delimiter used for encoding messages.
	 */
	public static final String DELIM = "$";

	/**
	 * Encode a list of arguments into one string.
	 * @param args An ArrayList of arguments
	 * @return The list of arguments as a string
	 */
	public static String encode(ArrayList<String> args) {
		StringBuilder sb = new StringBuilder();
		
		for (String s : args) {
			sb.append(s).append(DELIM);
		}
		
		return sb.toString();
	}
	
	/**
	 * Decode a message and put relevant arguments in a ArrayList.
	 * @param messageNumber A constant field value
	 * @param message A string message
	 * @return An ArrayList of arguments
	 */
	public static ArrayList<String> decode(int messageNumber, String message) {
		ArrayList<String> result = new ArrayList<String>();
		StringTokenizer toker = new StringTokenizer(message, DELIM);
				
		switch (messageNumber) {
		case MESSAGE_ATTACKER_CHOSE_SIDE :
			for (int i = 0 ; i < 2 ; i++) {
				if (toker.hasMoreTokens()) {
					result.add(toker.nextToken());
				}
			}
			break;
		case MESSAGE_ATTACKER_STEP :
			for (int i = 0 ; i < 3 ; i++) {
				if (toker.hasMoreTokens()) {
					result.add(toker.nextToken());
				}
			}
			break;
		case MESSAGE_DEFENDER_STEP :
			if (toker.hasMoreTokens()) {
				result.add(toker.nextToken());
			}
			break;
		case MESSAGE_RESET_GAME :
			break;
		default :
			break;
		}
		
		return result;		
	}
	
}
