package dk.brics.bisim.model;

import java.io.*;

/**
 * The user-definable properties.
 * @author Martin Mosegaard <a href="mailto:martinm@daimi.au.dk">martinm@daimi.au.dk</a>
 *
 */
public class UserProperties {

	private String dotPath;

	/**
	 * Calls readUserProperties. 
	 */
	public UserProperties() {
		dotPath = "";	
		readUserProperties();
	}	
	
	/**
	 * @return The dot path
	 */
	public String getDotPath() {
		return dotPath;
	}

	/**
	 * Reads properties from the file <tt>properties.txt</tt>
	 * if it exists in the current directory.
	 */
	private void readUserProperties() {
		BufferedReader reader = null;
		try {
			reader = new BufferedReader (new FileReader ("properties.txt"));
			String line;
			while ((line = reader.readLine()) != null) {
				
				/***** Switch on which property is being read *****/			
				if (line.startsWith( "dotPath" )) {
					/** Remove 'dotPath' and whitespace before '=' **/
					line = line.substring(7).trim();
					if (line.startsWith( "=" )) {
						/** Remove '=' and whitespace before the path **/
						dotPath = line.substring(1).trim();
					}
				}
								
			}
			reader.close();
		} catch (FileNotFoundException fnf) {
			System.err.println ("Could not read properties.txt file.");
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}

}
