package dk.brics.bisim.view;

import java.awt.Dimension;
import java.io.IOException;
import java.net.URL;

import javax.swing.JPanel;
import javax.swing.JTextPane;
import javax.swing.text.AbstractDocument;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.StyleConstants;

import dk.brics.bisim.JarResources;

/**
 * A panel displaying information and acknowledgements. The contents are read from a HTML file.
 * @author Martin Mosegaard <a href="mailto:martinm@daimi.au.dk">martinm@daimi.au.dk</a>
 *
 */
public class AboutView extends JPanel {

	private static final long serialVersionUID = 1L;

	private JTextPane textPane;
	
	public AboutView(int width, int height) {
		textPane = new JTextPane();
		textPane.setEditable(false);
		textPane.setSelectionColor(textPane.getBackground());
		URL pageUrl = getClass().getResource(JarResources.ABOUT_URL);
		if (pageUrl != null) {
			try {
				textPane.setPage(pageUrl);
			} catch (IOException e) {
			}
		}

		modifiedInsertIcon();	 
		
		textPane.setPreferredSize(new Dimension((75*width)/100, height));
		add(textPane);
	}

	/**
	 * This is a modification of JTextPane.insertIcon
	 * where the contents of StyleConstants.setIcon
	 * has been modified as suggested in the bug report with
	 * ID 4671653 at <a href="http://bugs.sun.com">http://bugs.sun.com</a>.
	 * Note: The bug is reportedly fixed in release 'mustang' (Java 6). 
	 */
	private void modifiedInsertIcon() {
		MutableAttributeSet inputAttributes = textPane.getInputAttributes();
		inputAttributes.removeAttributes(inputAttributes);

		/*** Modified code from StyleConstants.setIcon ***/
		inputAttributes.addAttribute(StyleConstants.NameAttribute, StyleConstants.IconElementName);
		inputAttributes.addAttribute(StyleConstants.IconAttribute, MainView.itVestImageIcon);	
		
		String content = " ";
		Document doc = textPane.getStyledDocument();
		if (doc != null) {
			try {
				if (doc instanceof AbstractDocument) {
					((AbstractDocument)doc).replace(0, 0, content, inputAttributes);
				}
			} catch (BadLocationException e) {
			}
		}		
		inputAttributes.removeAttributes(inputAttributes);
	}

}
