package dk.brics.bisim.view;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.KeyEvent;
import java.util.ArrayList;

import javax.swing.AbstractButton;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * The lower panel of a game view.
 * 
 * @author Martin Mosegaard <a href="mailto:martinm@daimi.au.dk">martinm@daimi.au.dk</a>
 *
 */
public class GameViewUpperPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	private JPanel logoPanel;
	private JLabel logoLabel;
	
	/**
	 * The chat panel. Not used in local games.
	 */
	private JPanel chatPanel;
	
	/**
	 * The button panel to the right, holding the reset and quit buttons.
	 */
	private JPanel buttonPanel;	
		
	private static final String chatString = "Chat: ";
	private static final int chatTextFieldSize = 50;
	public JTextField chatTextField;
	private static final String sendString = "Send";
	public JButton sendButton;
	private JPanel sendPanel;
	private JLabel chatMessageLabel;
	
	private static final String resetGameString = "Reset game";
	public JButton resetGameButton;
	private static final String quitString = "Quit game";
	public JButton quitButton;
	
	private int gameViewType;
	
	/**
	 * Constructs a lower panel based on a game type.
	 * @param gameViewType A game type
	 * @param width Width of the game view
	 * @param lowerHeight Height of this panel
	 */
	public GameViewUpperPanel(int gameViewType, int width, int lowerHeight) {	
		this.gameViewType = gameViewType;

		/*** Create logo panel ***/
		logoLabel = new JLabel(MainView.itVestImageIcon);
		logoPanel = new JPanel();
		logoPanel.add(logoLabel);
		
		/*** Create chat panel ***/
		chatPanel = new JPanel(new GridLayout(2,1));
		/*** A local game doesn't get a chat panel ***/
		if (gameViewType != GameView.GAME_VIEW_LOCAL) {
			chatTextField = new JTextField(chatTextFieldSize);
			sendButton = new JButton(sendString);
			sendPanel = new JPanel(new FlowLayout());
			sendPanel.add(new JLabel(chatString));
			sendPanel.add(chatTextField);
			sendPanel.add(sendButton);		
			chatMessageLabel = new JLabel();
			chatPanel.add(sendPanel);
			chatPanel.add(chatMessageLabel);
		}
		
		/*** Create button panel ***/
		quitButton = new JButton(quitString);
		quitButton.setMnemonic(KeyEvent.VK_Q);
		quitButton.setToolTipText("Quit and return to options menu");
		buttonPanel = new JPanel(new GridLayout(1,2));
		/*** A client doesn't get a reset game button ***/
		if (gameViewType != GameView.GAME_VIEW_CLIENT) {
				resetGameButton = new JButton(resetGameString);
				resetGameButton.setMnemonic(KeyEvent.VK_R);
				resetGameButton.setToolTipText("Reset the two LTSs");
				buttonPanel.add(resetGameButton);
		}
		buttonPanel.add(quitButton);

		/*** Create panel ***/
		setLayout(new BorderLayout());
		add(logoPanel,   BorderLayout.WEST);
		add(chatPanel,   BorderLayout.CENTER);
		add(buttonPanel, BorderLayout.EAST);
		setPreferredSize(new Dimension(width, lowerHeight));
	}
	
	/**
	 * Purpose: To return all buttons that need a remote action listener.
	 * @return An ArrayList of buttons
	 */
	public ArrayList<AbstractButton> getButtons() {
		ArrayList<AbstractButton> buttons = new ArrayList<AbstractButton>();
		
		/*** A client can't use the reset button ***/
		if (gameViewType != GameView.GAME_VIEW_CLIENT)
			buttons.add(resetGameButton);
		/*** A local game doesn't have a send button for chat messages ***/
		if (gameViewType != GameView.GAME_VIEW_LOCAL)
			buttons.add(sendButton);
		
		buttons.add(quitButton);
		
		return buttons;
	}

	/**
	 * Purpose: To return all text fields that need an action listener.
	 * @return An ArrayList of text fields
	 */
	public ArrayList<JTextField> getTextFields() {
		ArrayList<JTextField> textFields = new ArrayList<JTextField>();
		
		if (gameViewType != GameView.GAME_VIEW_LOCAL)
			textFields.add(chatTextField);
		
		return textFields;		
	}

	/**
	 * @return The label holding a chat message
	 */
	public JLabel getChatMessageLabel() {
		return chatMessageLabel;
	}
	
}
