package dk.brics.bisim.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Image;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.net.URL;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import dk.brics.bisim.JarResources;

/**
 * A panel describing game rules and help. The contents are read from a HTML file.
 * @author Martin Mosegaard <a href="mailto:martinm@daimi.au.dk">martinm@daimi.au.dk</a>
 *
 */
public class HelpView extends JPanel {

	private static final long serialVersionUID = 1L;

	private JEditorPane textPane;
	private JScrollPane scrollPane;
	private JPanel picturePanel;
	private Image gameImage;
	private ImageIcon gameImageIcon;
	private JLabel pictureLabel;
	
	public HelpView(int width, int height) {
		textPane = new JEditorPane();
		textPane.setEditable(false);
		textPane.setSelectionColor(textPane.getBackground());
		URL pageUrl = getClass().getResource(JarResources.HELP_URL);
		if (pageUrl != null) {
			try {
				textPane.setPage(pageUrl);
			} catch (IOException e) {
			}
		}
		
		Dimension wdim = new Dimension((25*width)/100, height);
		Dimension edim = new Dimension((75*width)/100, height);
 		
		/** Create game picture **/
		byte[] buffer = JarResources.getResource(JarResources.JAR_FILE, JarResources.GAME_FILE);
	    try {
	        gameImage = ImageIO.read(new ByteArrayInputStream(buffer));
	    } catch (IOException e) {
	    }
        if (gameImage != null)
        	gameImageIcon = new ImageIcon(gameImage);

		pictureLabel = new JLabel(gameImageIcon);
		picturePanel = new JPanel(new GridLayout(1,1));
		picturePanel.setBackground(Color.WHITE);
		picturePanel.add(pictureLabel);
		picturePanel.setPreferredSize(wdim);

		scrollPane = new JScrollPane(textPane);
		scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setPreferredSize(edim);		
		
		setLayout(new BorderLayout());
		add(picturePanel, BorderLayout.WEST);
		add(scrollPane,   BorderLayout.EAST);
	}
	
}
