package dk.brics.bisim.view;

import java.awt.BorderLayout;
import java.awt.Color;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;

import java.awt.Dimension;

import att.grappa.Graph;
import att.grappa.GrappaAdapter;
import att.grappa.GrappaPanel;

/**
 * A panel for displaying a LTS. It contains a titled border,
 * a button panel, and a scroll pane.
 * 
 * @author Martin Mosegaard <a href="mailto:martinm@daimi.au.dk">martinm@daimi.au.dk</a>
 *
 */
public class LTSPanel extends JPanel {
	
	private static final long serialVersionUID = 1L;
	
	/**
	 * A scroll pane holding a grappa panel. 
	 */
	private JScrollPane scrollPane; 
	
	/**
	 * A grappa panel displaying a graph.
	 */
	private GrappaPanel grappaPanel;
	
	/**
	 * The line border of the titled border. 
	 */
	private Border lineBorder;
	
	/**
	 * The titled border.
	 */
	private TitledBorder titledBorder;
	
	/**
	 * The title of the border.
	 */
	private String title;
	
	/**
	 * Whether or not the border is red.
	 */
	private boolean borderIsRed;
	
	/**
	 * Constructs an LTS panel given the preferred dimension of a graph panel.
	 * @param graphDimension Graph dimension
	 */
	public LTSPanel(Dimension graphDimension) {
		/*** Create center panel ***/
		scrollPane = new JScrollPane ();
	    scrollPane.getViewport().setBackground(Color.white);
	    scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
	    scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);

	    /*** Create border ***/
	    lineBorder = BorderFactory.createLineBorder(Color.LIGHT_GRAY, 2);
	    titledBorder = BorderFactory.createTitledBorder(lineBorder);
	    titledBorder.setTitleJustification(TitledBorder.CENTER);
	    setBorder(titledBorder);
	    	
	    setPreferredSize(graphDimension);
	    setLayout(new BorderLayout());
	    add(scrollPane, BorderLayout.CENTER);
	}

	/**
	 * Display the LTS.
	 * @param graph The graph to display
	 * @param ga A Grappa adapter
	 */
	public void displayLTS(Graph graph, GrappaAdapter ga) {
		grappaPanel = new GrappaPanel(graph);
	    grappaPanel.setScaleToFit(false);
	    grappaPanel.addGrappaListener(ga);
	    scrollPane.setViewportView(grappaPanel);
	}

	/**
	 * @return The Grappa panel
	 */
	public GrappaPanel getGrappaPanel() {
		return grappaPanel;
	}	

	/**
	 * @return The scroll pane
	 */
	public JScrollPane getScrollPane() {
		return scrollPane;
	}

	/**
	 * Set the title of the panel.
	 * @param title A title
	 */
	public void setTitle(String title) {
		this.title = title;
		titledBorder.setTitle(title);
	}
	
	/**
	 * Toggle border color (red / light gray).
	 */
	public void toggleBorder() {
		if (borderIsRed) {
		    lineBorder = BorderFactory.createLineBorder(Color.LIGHT_GRAY, 2);
		}
		else {
			lineBorder = BorderFactory.createLineBorder(Color.RED, 2);
		}
	    titledBorder = BorderFactory.createTitledBorder(lineBorder);
	    titledBorder.setTitleJustification(TitledBorder.CENTER);			
		titledBorder.setTitle(title);
	    setBorder(titledBorder);
	    borderIsRed = !borderIsRed;
	}
	
	/**
	 * Reset border color to light gray.
	 */
	public void resetBorder() {
		borderIsRed = true;
		toggleBorder();
	}
	
}
