package dk.brics.bisim.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.event.KeyEvent;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import javax.imageio.ImageIO;
import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.border.Border;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

import dk.brics.bisim.JarResources;



/**
 * The view for selecting game options.
 * 
 * @author Martin Mosegaard <a href="mailto:martinm@daimi.au.dk">martinm@daimi.au.dk</a>
 *
 */
public class OptionsView extends JPanel {
	
	private static final long serialVersionUID = 1L;
	
	/**
	 * The main panel of the options view.
	 */
	private JPanel mainPanel;

	/**
	 * The image panel to the left.
	 */
	private JPanel imagePanel;
	
	/**
	 * The tabbed pane to the right.
	 */
	private JTabbedPane tabbedPane;
	
	/**
	 * The status panel underneath the tabbed pane.
	 */
	private JPanel lowerPanel;
	
	private JLabel imageLabel;
	private Image image;
	private ImageIcon icon;
	
	private JPanel gameTypePanel;
	private static final String gameTypeString = "Game type";

	private JPanel playersPanel;
	private static final String playersString = "Players";
	private ButtonGroup playersButtonGroup;
	private static final String onePlayerString = "One player";
	public JRadioButton onePlayerRadioButton;
	private static final String twoPlayerLocalString = "Two player local";
	public JRadioButton twoPlayerLocalRadioButton;
	private static final String twoPlayerRemoteString = "Two player remote";
	public JRadioButton twoPlayerRemoteRadioButton;

	private JPanel clientServerPanel;
	private static final String clientServerString = "Start a";
	private ButtonGroup clientServerButtonGroup;
	private static final String serverString = "Server";
	public JRadioButton serverRadioButton;
	private static final String clientString = "Client";
	public JRadioButton clientRadioButton;
	
	private JPanel serverSettingsPanel;
	private static final String serverSettingsString = "Server settings";
	private static final String hostString = "Host: ";
	private JLabel hostLabel;
	public JTextField hostTextField;
	private static final int hostTextFieldSize = 20;
	private static final String applyString = "Apply";
	public JButton applyButton;
	
	private JPanel rolePanel;
	private static final String roleString = "Player role";
	private ButtonGroup roleButtonGroup;
	private static final String attackerString = "Attacker";
	public JRadioButton attackerRadioButton;
	private static final String defenderString = "Defender";
	public JRadioButton defenderRadioButton;

	private Border loweredetched;
	private TitledBorder playersBorder;
	private TitledBorder roleBorder;
	private TitledBorder serverSettingsBorder;
	private TitledBorder clientServerBorder;
	
	private JPanel ltsPanel;
	public static final String ltsString = "LTS";
	private static final String browseString = "Browse...";
	private static final int ltsTextFieldSize = 35;
	private static final String leftLTSString = "Left LTS:";
	private JLabel leftLTSLabel;
	private JPanel leftLTSPanel;
	private JTextField leftLTSTextField;
	public JButton leftLTSBrowseButton;
	private static final String rightLTSString = "Right LTS:";
	private JLabel rightLTSLabel;
	private JPanel rightLTSPanel;
	private JTextField rightLTSTextField;
	public JButton rightLTSBrowseButton;
	
	private JPanel bisimulationPanel;
	public static final String bisimulationString = "Bisimulation";
	private ButtonGroup bisimulationButtonGroup;
	private static final String strongString = "Strong";
	public JRadioButton strongRadioButton;
	private static final String weakString = "Weak";
	public JRadioButton weakRadioButton;
	
	private JPanel statusPanel;
	private JLabel statusLabel;
	private JPanel buttonPanel;
	private static final String startString = "Start game";
	public JButton startButton;
	private static final String exitString = "Exit";
	public JButton exitButton;
	
	private File currentDir;
	
	public OptionsView() {
		currentDir = new File(System.getProperty("user.dir"));
		
		createImagePanel();	
		createTabbedPane();
		createLowerPanel();
		
		mainPanel = new JPanel(new BorderLayout());
		mainPanel.add(tabbedPane, BorderLayout.CENTER);
		mainPanel.add(lowerPanel, BorderLayout.SOUTH);

		setLayout(new FlowLayout());
		add(imagePanel);
		add(mainPanel);
	}

	private void createImagePanel() {
		image = null;

		byte[] amazingBuffer = JarResources.getResource(JarResources.JAR_FILE, JarResources.SPLASH_FILE);
	    try {
	        image = ImageIO.read(new ByteArrayInputStream(amazingBuffer));
	    } catch (IOException e) {
	    }
        imagePanel = new JPanel();
        icon = null;
        if (image != null) {
        	icon = new ImageIcon(image);
        }
	    imageLabel = new JLabel(icon);
	    imagePanel.add(imageLabel);
	}
	
	private void createTabbedPane() {
		createGameTypePanel();	
		createLTSPanel();
		createBisimulationPanel();
	
		/*** create tabbed pane ***/
		tabbedPane = new JTabbedPane();
		tabbedPane.add(gameTypeString, gameTypePanel);
		tabbedPane.add(ltsString, ltsPanel);
		tabbedPane.add(bisimulationString, bisimulationPanel);

		int gameTypeIndex = tabbedPane.indexOfTab(gameTypeString);
		int ltsIndex = tabbedPane.indexOfTab(ltsString);
		int bisimulationIndex = tabbedPane.indexOfTab(bisimulationString);
		
		tabbedPane.setMnemonicAt(gameTypeIndex, KeyEvent.VK_G);
		tabbedPane.setMnemonicAt(ltsIndex, KeyEvent.VK_L);
		tabbedPane.setMnemonicAt(bisimulationIndex, KeyEvent.VK_B);
		
		tabbedPane.setEnabledAt(ltsIndex, false);
		tabbedPane.setEnabledAt(bisimulationIndex, false);
	}


	private void createGameTypePanel() {
	    loweredetched = BorderFactory.createEtchedBorder(EtchedBorder.LOWERED);

	    /*** Players submenu ***/
		onePlayerRadioButton = new JRadioButton(onePlayerString);
		onePlayerRadioButton.setMnemonic(KeyEvent.VK_1);
		twoPlayerLocalRadioButton = new JRadioButton(twoPlayerLocalString);
		twoPlayerLocalRadioButton.setMnemonic(KeyEvent.VK_2);
		twoPlayerRemoteRadioButton = new JRadioButton(twoPlayerRemoteString);
		twoPlayerRemoteRadioButton.setMnemonic(KeyEvent.VK_3);
		playersButtonGroup = new ButtonGroup();
		playersButtonGroup.add(onePlayerRadioButton);
		playersButtonGroup.add(twoPlayerLocalRadioButton);
		playersButtonGroup.add(twoPlayerRemoteRadioButton);
		playersPanel = new JPanel(new GridLayout(3,1));
		playersPanel.add(onePlayerRadioButton);
		playersPanel.add(twoPlayerLocalRadioButton);
		playersPanel.add(twoPlayerRemoteRadioButton);
	    playersBorder = BorderFactory.createTitledBorder(loweredetched);
	    playersBorder.setTitleJustification(TitledBorder.LEFT);
	    playersBorder.setTitle(playersString);
	    playersPanel.setBorder(playersBorder);		

		/*** Client/server submenu ***/
		serverRadioButton = new JRadioButton(serverString);
		serverRadioButton.setMnemonic(KeyEvent.VK_S);
		serverRadioButton.setEnabled(false);
		clientRadioButton = new JRadioButton(clientString);
		clientRadioButton.setMnemonic(KeyEvent.VK_C);
		clientRadioButton.setEnabled(false);
		clientServerButtonGroup = new ButtonGroup();
		clientServerButtonGroup.add(serverRadioButton);
		clientServerButtonGroup.add(clientRadioButton);		
		clientServerPanel = new JPanel (new GridLayout(2,1));
		clientServerPanel.add(serverRadioButton);
		clientServerPanel.add(clientRadioButton);		
	    clientServerBorder = BorderFactory.createTitledBorder(loweredetched);
	    clientServerBorder.setTitleJustification(TitledBorder.LEFT);
	    clientServerBorder.setTitle(clientServerString);
	    clientServerPanel.setBorder(clientServerBorder);
		
	    /*** Server settings submenu ***/
		hostLabel = new JLabel(hostString);
		hostLabel.setEnabled(false);
		hostTextField = new JTextField(hostTextFieldSize);
		hostTextField.setEnabled(false);
		applyButton = new JButton(applyString);
		applyButton.setEnabled(false);
		serverSettingsPanel = new JPanel(new FlowLayout());
		serverSettingsPanel.add(hostLabel);
		serverSettingsPanel.add(hostTextField);
		serverSettingsPanel.add(applyButton);
	    serverSettingsBorder = BorderFactory.createTitledBorder(loweredetched);
	    serverSettingsBorder.setTitleJustification(TitledBorder.LEFT);
	    serverSettingsBorder.setTitle(serverSettingsString);
	    serverSettingsPanel.setBorder(serverSettingsBorder);
	    
		/*** Player role submenu ***/
		attackerRadioButton = new JRadioButton(attackerString);
		attackerRadioButton.setMnemonic(KeyEvent.VK_A);
		attackerRadioButton.setEnabled(false);
		defenderRadioButton = new JRadioButton(defenderString);
		defenderRadioButton.setMnemonic(KeyEvent.VK_D);
		defenderRadioButton.setEnabled(false);
		roleButtonGroup = new ButtonGroup();
		roleButtonGroup.add(attackerRadioButton);
		roleButtonGroup.add(defenderRadioButton);		
		rolePanel = new JPanel (new GridLayout(2,1));
		rolePanel.add(attackerRadioButton);
		rolePanel.add(defenderRadioButton);
	    roleBorder = BorderFactory.createTitledBorder(loweredetched);
	    roleBorder.setTitleJustification(TitledBorder.LEFT);
	    roleBorder.setTitle(roleString);
	    rolePanel.setBorder(roleBorder);
	    
		/*** Add components ***/
		gameTypePanel = new JPanel(new GridLayout(4,1));
		gameTypePanel.add(playersPanel);
		gameTypePanel.add(clientServerPanel);
		gameTypePanel.add(serverSettingsPanel);
		gameTypePanel.add(rolePanel);
	}
	

	private void createLTSPanel() {
		leftLTSLabel = new JLabel(leftLTSString);
		leftLTSTextField = new JTextField(ltsTextFieldSize);
		leftLTSTextField.setEditable(false);
		leftLTSTextField.setBackground(Color.white);

		leftLTSBrowseButton = new JButton(browseString);
		leftLTSBrowseButton.setMnemonic(KeyEvent.VK_LEFT);
		leftLTSBrowseButton.setToolTipText("Choose left LTS");

		leftLTSPanel = new JPanel (new FlowLayout());
		leftLTSPanel.add(leftLTSTextField);
		leftLTSPanel.add(leftLTSBrowseButton);
		
		rightLTSLabel = new JLabel(rightLTSString);
		rightLTSTextField = new JTextField(ltsTextFieldSize);
		rightLTSTextField.setEditable(false);
		rightLTSTextField.setBackground(Color.white);
		
		rightLTSBrowseButton = new JButton(browseString);
		rightLTSBrowseButton.setMnemonic(KeyEvent.VK_RIGHT);
		rightLTSBrowseButton.setToolTipText("Choose right LTS");
		
		rightLTSPanel = new JPanel (new FlowLayout());		
		rightLTSPanel.add(rightLTSTextField);
		rightLTSPanel.add(rightLTSBrowseButton);
		
		ltsPanel = new JPanel(new GridLayout(4,1));
		ltsPanel.add(leftLTSLabel);
		ltsPanel.add(leftLTSPanel);
		ltsPanel.add(rightLTSLabel);
		ltsPanel.add(rightLTSPanel);
	}

	
	private void createBisimulationPanel() {
		strongRadioButton = new JRadioButton(strongString);
		strongRadioButton.setMnemonic(KeyEvent.VK_S);
		weakRadioButton = new JRadioButton(weakString);
		weakRadioButton.setMnemonic(KeyEvent.VK_W);
		
		bisimulationButtonGroup = new ButtonGroup();
		bisimulationButtonGroup.add(strongRadioButton);
		bisimulationButtonGroup.add(weakRadioButton);
		
		bisimulationPanel = new JPanel(new GridLayout(2,1));
		bisimulationPanel.add(strongRadioButton);
		bisimulationPanel.add(weakRadioButton);
	}
	
		
	private void createLowerPanel() {
		statusLabel = new JLabel();
		statusPanel = new JPanel(new FlowLayout());
		statusPanel.add(statusLabel);
		
		startButton = new JButton(startString);
		startButton.setEnabled(false);
		startButton.setMnemonic(KeyEvent.VK_ENTER);
		startButton.setToolTipText("Start the game");
		
		exitButton = new JButton(exitString);
		exitButton.setMnemonic(KeyEvent.VK_E);
		exitButton.setToolTipText("Exit");
		
		buttonPanel = new JPanel(new FlowLayout());
		buttonPanel.add(startButton);
		buttonPanel.add(exitButton);
		
		lowerPanel = new JPanel (new BorderLayout());
		lowerPanel.add(statusPanel, BorderLayout.WEST);
		lowerPanel.add(buttonPanel, BorderLayout.EAST);
	}
	
	
	/**
	 * Purpose: To return all buttons that need an action listener.
	 * @return An ArrayList of buttons
	 */
	public ArrayList<AbstractButton> getButtons() {
		ArrayList<AbstractButton> buttons = new ArrayList<AbstractButton>();
		
		buttons.add(onePlayerRadioButton);
		buttons.add(twoPlayerLocalRadioButton);
		buttons.add(twoPlayerRemoteRadioButton);
		buttons.add(serverRadioButton);
		buttons.add(clientRadioButton);
		buttons.add(applyButton);
		buttons.add(leftLTSBrowseButton);
		buttons.add(rightLTSBrowseButton);
		buttons.add(strongRadioButton);
		buttons.add(weakRadioButton);
		buttons.add(attackerRadioButton);
		buttons.add(defenderRadioButton);
		buttons.add(startButton);
		buttons.add(exitButton);
		
		return buttons;
	}
	
	
	/**
	 * Purpose: To return all text fields that need an action listener.
	 * @return An ArrayList of text fields
	 */
	public ArrayList<JTextField> getTextFields() {
		ArrayList<JTextField> textFields = new ArrayList<JTextField>();
		
		textFields.add(hostTextField);
		
		return textFields;		
	}
	
	/**
	 * Set the path to the left LTS.
	 * @param path A path name
	 */
	public void setLeftLTSPath(String path) {
		leftLTSTextField.setText(path);
	}

	/**
	 * Set the path to the right LTS.
	 * @param path A path name
	 */
	public void setRightLTSPath(String path) {
		rightLTSTextField.setText(path);
	}
	
	/**
	 * Open a file chooser and return the file path.
	 */
	public File chooseLTS() {
		File file = null;
		
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setCurrentDirectory(currentDir);
		fileChooser.setFileFilter(new GraphFileFilter());
		int returnVal = fileChooser.showDialog(this, null);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			currentDir = fileChooser.getCurrentDirectory();
			file = fileChooser.getSelectedFile();
		}

		return file;
	}

	/**
	 * Enable buttons inside role panel.
	 * @param enable Whether or not to enable
	 */
	public void setRolePanelEnabled(boolean enable) {
		attackerRadioButton.setEnabled(enable);
		defenderRadioButton.setEnabled(enable);
	}
	
	/**
	 * Enable/disable text fields and buttons for
	 * remote settings.
	 * @param enable Whether or not to enable
	 */
	public void setRemotePanelEnabled(boolean enable) {
		serverRadioButton.setEnabled(enable);
		clientRadioButton.setEnabled(enable);
	}
	
	/**
	 * Enable/disable components inside the host panel.
	 * @param enable Whether or not to enable
	 */
	public void setHostPanelEnabled(boolean enable) {
		hostLabel.setEnabled(enable);
		hostTextField.setEnabled(enable);	
		applyButton.setEnabled(enable);
	}
	
	/**
	 * Enable/disable a tab in the tabbed pane.
	 * @param tabTitle Title of the tab
	 * @param enable Whether or not to enable
	 */
	public void setTabEnabled(String tabTitle, boolean enable) {
		tabbedPane.setEnabledAt(tabbedPane.indexOfTab(tabTitle), enable);
	}
	
	/**
	 * Display a status in the lower panel.
	 * @param mes A status message
	 */
	public void displayStatus(String mes) {
		statusLabel.setText(mes);
	}
	
}
