package dk.brics.bisim.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.KeyEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.util.HashMap;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import att.grappa.Graph;
import att.grappa.GrappaAdapter;
import att.grappa.GrappaPanel;

/**
 * A panel for the CCS Visualizer and Simulator.
 * @author Martin Mosegaard <a href="mailto:martinm@daimi.au.dk">martinm@daimi.au.dk</a>
 *
 */
public class SimulatorView extends JPanel {

	private static final long serialVersionUID = 1L;

	/**
	 * Minimum number of actions to make room for in the actions 
	 * panel to the west.
	 */
	private static final int NO_OF_ACTIONS = 10;

	private int width, height;
	private Dimension graphDimension;
	private Dimension actionDimension;
	private Dimension stepButtonDimension;
	private Dimension traceDimension;
	public JButton openFileButton;
	public JButton resetButton;
	public JButton exitButton;
	private JPanel buttonPanel;
	private JScrollPane graphScrollPane;
	private JScrollPane traceScrollPane;
	private JPanel actionPanel;
	private JScrollPane inputActionScrollPane;
	private JPanel inputActions;
	private JPanel inputPanel;
	public JButton inputButton;
	public JButton tauButton;
	public JButton randomButton;
	public JButton outputButton;
	private JPanel stepButtonPanel;
	private JScrollPane outputActionScrollPane;
	private JPanel outputActions;
	private JPanel outputPanel;
	private JTextArea trace;

	private JFileChooser fileChooser;

	private File currentDir;

	private GrappaPanel gp;
	
	public SimulatorView(int width, int height) {
		this.width = width;
		this.height = height;
		setDimensions();
		createAndShow();
	}
	
	private void setDimensions() {
		graphDimension = new Dimension(width-290, height);
		actionDimension = new Dimension(180, height/2 - 60);
		stepButtonDimension = new Dimension(180, 105);
		traceDimension = new Dimension(100, height);
		setSize(width, height);
	}

	private void createAndShow() {
		/***** South - Button panel *****/
	    createLowerButtons();
	    
	    /***** Center - Scroll pane for graphs *****/
	    graphScrollPane = new JScrollPane ();
	    graphScrollPane.getViewport().setBackground(Color.white);
	    graphScrollPane.setPreferredSize(graphDimension);
	    graphScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
	    graphScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
	    	    
	    /***** West - Action panel *****/
	    createActionPanel();
	    
	    /***** East - Trace display *****/
	    traceScrollPane = new JScrollPane ();
	    traceScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
	    traceScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	    resetTrace();
	    
	    /***** Place components *****/
	    setLayout(new BorderLayout());
	    setOpaque(true);
 		add(buttonPanel, BorderLayout.SOUTH);
		add(graphScrollPane, BorderLayout.CENTER);
		add(traceScrollPane, BorderLayout.EAST);
		add(actionPanel, BorderLayout.WEST);
	}

	private void createLowerButtons() {
		openFileButton = new JButton("Open file..");
		openFileButton.setMnemonic(KeyEvent.VK_PERIOD);
		openFileButton.setToolTipText("Read graph from a CWB output file");
		
	    resetButton = new JButton("Reset");
	    resetButton.setMnemonic(KeyEvent.VK_R);
		resetButton.setToolTipText("Reset automaton");
		resetButton.setEnabled(false);	
		
		exitButton = new JButton("Exit");
		exitButton.setMnemonic(KeyEvent.VK_E);
		exitButton.setToolTipText("Exit");

	    buttonPanel = new JPanel();
	    buttonPanel.add(openFileButton);
	    buttonPanel.add(resetButton);
	    buttonPanel.add(exitButton);
	}

	private void createActionPanel() {
		/***** Input panel *****/
		inputActionScrollPane = new JScrollPane();
	    inputActionScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
	    inputActionScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		inputActions = new JPanel();
		inputActions.setBackground(Color.white);
	    inputActionScrollPane.setPreferredSize(actionDimension);
	    inputActionScrollPane.setViewportView(inputActions);	    
	    inputPanel = new JPanel(new BorderLayout());
	    inputPanel.add(new JLabel("Input actions:"), BorderLayout.NORTH);
	    inputPanel.add(inputActionScrollPane, BorderLayout.CENTER);
	    
	    /***** Button panel *****/
	    inputButton = new JButton("Input");
		inputButton.setMnemonic(KeyEvent.VK_I);
		inputButton.setToolTipText("Fire input action");
		inputButton.setEnabled(false);

	    tauButton = new JButton("Tau");
		tauButton.setMnemonic(KeyEvent.VK_T);
		tauButton.setToolTipText("Fire tau action");
		tauButton.setEnabled(false);	

	    randomButton = new JButton("Random");
		randomButton.setMnemonic(KeyEvent.VK_ENTER);
		randomButton.setToolTipText("Fire random action");
		randomButton.setEnabled(false);	

	    outputButton = new JButton("Output");
		outputButton.setMnemonic(KeyEvent.VK_O);
		outputButton.setToolTipText("Fire output action");
		outputButton.setEnabled(false);	

	    stepButtonPanel = new JPanel(new GridLayout(3,2));
	    stepButtonPanel.setPreferredSize(stepButtonDimension);
	    stepButtonPanel.add(new JPanel());
	    stepButtonPanel.add(inputButton);
	    stepButtonPanel.add(tauButton);
	    stepButtonPanel.add(randomButton);
	    stepButtonPanel.add(new JPanel());
	    stepButtonPanel.add(outputButton);
	    
	    /***** Output panel *****/
	    outputActionScrollPane = new JScrollPane();
	    outputActionScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
	    outputActionScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		outputActions = new JPanel();
		outputActions.setBackground(Color.white);
	    outputActionScrollPane.setPreferredSize(actionDimension);
	    outputActionScrollPane.setViewportView(outputActions);
	    outputPanel = new JPanel(new BorderLayout());
	    outputPanel.add(new JLabel("Output actions:"), BorderLayout.NORTH);
	    outputPanel.add(outputActionScrollPane, BorderLayout.CENTER);
	    
	    /***** Place components *****/
	    actionPanel = new JPanel(new BorderLayout());
	    actionPanel.add(inputPanel, BorderLayout.NORTH);
	    actionPanel.add(stepButtonPanel, BorderLayout.CENTER);
	    actionPanel.add(outputPanel, BorderLayout.SOUTH);
	}
	
	public void resetTrace() {
	    trace = new JTextArea("Trace:");
	    trace.setEditable(false);
	    traceScrollPane.setPreferredSize(traceDimension);
	    traceScrollPane.setViewportView(trace);
	}
	
	public void updateTrace(String action) {
		trace.append("\n");
		trace.append(action);
	}
	
	/**
	 * @return A list of buttons that need an action listener
	 */
	public ArrayList<JButton> getButtons() {
		ArrayList<JButton> buttons = new ArrayList<JButton>();
		
		buttons.add(inputButton);		
		buttons.add(tauButton);
		buttons.add(randomButton);
		buttons.add(outputButton);
		buttons.add(openFileButton);		
		buttons.add(resetButton);
		buttons.add(exitButton);
		
		return buttons;
	}
	
	public void showActions(HashMap<String,String> inputActionMap,
							HashMap<String,String> outputActionMap,
							MouseListener mListen) {
		
		resetInputActions(inputActionMap.size());
		for (String name : inputActionMap.keySet()) {
			JCheckBox cb = new JCheckBox(inputActionMap.get(name));
			cb.setName(name);
			cb.setBackground(Color.white);
			cb.addMouseListener(mListen);
			inputActions.add(cb);
		}
		resetOutputActions(outputActionMap.size());
		for (String name : outputActionMap.keySet()) {
			JCheckBox cb = new JCheckBox(outputActionMap.get(name));
			cb.setName(name);
			cb.setBackground(Color.white);
			cb.addMouseListener(mListen);
			outputActions.add(cb);
		}
	}
	
	private void resetInputActions(int elements) {
		if (elements < NO_OF_ACTIONS) {
			inputActions = new JPanel(new GridLayout(NO_OF_ACTIONS, 1));
		}
		else {
			inputActions = new JPanel(new GridLayout(elements, 1));
		}
		inputActions.setBackground(Color.white);
	    inputActionScrollPane.setPreferredSize(actionDimension);
	    inputActionScrollPane.setViewportView(inputActions);
	}
	
	private void resetOutputActions(int elements) {
		if (elements < NO_OF_ACTIONS) {
			outputActions = new JPanel(new GridLayout(NO_OF_ACTIONS, 1));
		}
		else {
			outputActions = new JPanel(new GridLayout(elements, 1));
		}
		outputActions.setBackground(Color.white);
	    outputActionScrollPane.setPreferredSize(actionDimension);
	    outputActionScrollPane.setViewportView(outputActions);
	}
	
	public File openFile () {
		File file = null;
		fileChooser = new JFileChooser ();
		fileChooser.setCurrentDirectory(currentDir);
		int returnVal = fileChooser.showDialog(this, null);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			currentDir = fileChooser.getCurrentDirectory();
			file = fileChooser.getSelectedFile();
		}
		return file;
	}
		
	public void showGraph (Graph graph, GrappaAdapter ga) {
		gp = new GrappaPanel(graph);
	    gp.setScaleToFit(false);
	    gp.addGrappaListener(ga);
	    graphScrollPane.setViewportView(gp);
	}

}
